@@ -1,6 +1,6 @@
 /* -*-comment-start: "//";comment-end:""-*-
  * GNU Mes --- Maxwell Equations of Software
- * Copyright © 2017 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+ * Copyright © 2017,2018 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
  *
  * This file is part of GNU Mes.
  *
@@ -28,14 +28,20 @@
 #include_next <errno.h>
 #else // ! WITH_GLIBC
 
+#ifndef __MES_ERRNO_T
+#define __MES_ERRNO_T 1
 typedef int error_t;
+int errno;
+#endif // !__MES_ERRNO_T
 
 int errno;
 #define ENOENT   2
 #define EINTR    4
 #define EIO      5
 #define E2BIG    7
+#define	ENOEXEC  8
 #define EBADF    9
+#define	ECHILD  10
 #define EAGAIN  11
 #define ENOMEM  12
 #define EEXIST  17
