@@ -29,13 +29,14 @@
 
 #else // ! WITH_GLIBC
 
-#define O_RDONLY 0
-#define O_WRONLY 1
-#define O_RDWR 2
-#define O_CREAT 64
-#define O_EXCL 128
-#define O_TRUNC 512
-#define O_APPEND 1024
+#define O_RDONLY          0
+#define O_WRONLY          1
+#define O_RDWR            2
+#define O_CREAT        0x40
+#define O_EXCL         0x80
+#define O_TRUNC       0x200
+#define O_APPEND      0x400
+#define O_DIRECTORY 0x10000
 
 #define FD_CLOEXEC 1
 
