@@ -21,22 +21,26 @@
 #ifndef __MES_LIBMES_H
 #define __MES_LIBMES_H
 
+#include <libmes-mini.h>
+
 int __mes_debug ();
-char const* number_to_ascii (int number, int base, int signed_p);
+long abtol (char const** p, int base);
+char const* ntoab (long number, int base, int signed_p);
 char const* itoa (int number);
-char const* utoa (unsigned number);
-char const* itoab (int x, int base);
-int _atoi (char const**, int base);
+char const* utoa (unsigned long number);
+char const* ltoab (long x, int base);
 int atoi (char const *s);
 int eputc (int c);
-int eputs (char const* s);
 int fdgetc (int fd);
 int fdputc (int c, int fd);
 int fdputs (char const* s, int fd);
 int fdungetc (int c, int fd);
+int _fdungetc_p (int fd);
 int isdigit (int c);
 int isspace (int c);
 int isxdigit (int c);
 int oputs (char const* s);
+ssize_t write (int filedes, void const *buffer, size_t size);
+char *search_path (char const *file_name);
 
 #endif //__MES_LIBMES_H
