@@ -18,15 +18,25 @@
  * along with GNU Mes.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <errno.h>
+
 #if __MESC__
 
-#include <linux/mini-mes.c>
+#include <linux/x86-mes/mini.c>
 
-#else // !__MESC__
+#elif __i386__
 
-#include <linux/mini-gcc.c>
+#include <linux/x86-mes-gcc/mini.c>
 
-#endif // !__MESC__
+#elif __x86_64__
+
+#include <linux/x86_64-mes-gcc/mini.c>
+
+#else
+
+#error arch not supported
+
+#endif
 
 ssize_t
 write (int filedes, void const *buffer, size_t size)
