@@ -18,11 +18,10 @@
  * along with GNU Mes.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "30-test.i"
-#include <stdio.h>
+#include <libmes-mini.h>
 
 int
-test ()
+main ()
 {
   int f = 0;
   int t = 1;
