@@ -18,13 +18,13 @@
  * along with GNU Mes.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "30-test.i"
+#include <libmes.h>
 #include <libmes.h>
 #include <stdlib.h>
 #include <string.h>
 
 int
-test ()
+main ()
 {
   char *p = "mes";
 
