@@ -18,7 +18,7 @@
  * along with GNU Mes.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "30-test.i"
+#include <libmes.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -32,7 +32,7 @@ int global_i = 1;
 int *global_p = &global_i;
 
 int
-test ()
+main ()
 {
   if (printf ("belong to us\n"), strcmp (help, "All your base are")) return 1;
 
