@@ -18,7 +18,7 @@
  * along with GNU Mes.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "30-test.i"
+#include <libmes.h>
 #include <stdio.h>
 
 struct s
@@ -42,7 +42,7 @@ struct anon
 
 
 int
-test ()
+main ()
 {
   struct foo f = {1,2};
   f.s.baz = 2; // FIXME
