@@ -18,17 +18,17 @@
  * along with GNU Mes.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "30-test.i"
+#include <libmes.h>
 #include <libmes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 int
-test ()
+main ()
 {
   char *p = "42foo\n";
-  int n = abtoi (&p, 0);
+  int n = abtol (&p, 0);
   if (n != 42) return 1;
   eputs (p);
   if (strcmp (p, "foo\n")) return 2;
