@@ -18,12 +18,12 @@
  * along with GNU Mes.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "30-test.i"
+#include <libmes.h>
 #include <stdio.h>
 #include <string.h>
 
 int
-test ()
+main ()
 {
   puts ("\n");
   char buf[10] = {0,0,0,0,0,0,0,0,0,0};
